package ru.dbotthepony.mc.otm.block.entity.matter

import net.minecraft.core.BlockPos
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.block.matter.PatternStorageBlock
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import ru.dbotthepony.mc.otm.menu.matter.PatternStorageMenu
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import ru.dbotthepony.kommons.collect.filterNotNull
import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.capability.matter.*
import ru.dbotthepony.mc.otm.container.slotted.ContainerSlot
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.kommons.collect.map
import ru.dbotthepony.mc.otm.util.isNotEmpty
import ru.dbotthepony.mc.otm.graph.matter.SimpleMatterNode
import ru.dbotthepony.mc.otm.registry.game.MBlockEntities
import java.util.stream.Stream

class PatternStorageBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryDeviceBlockEntity(MBlockEntities.PATTERN_STORAGE, p_155229_, p_155230_), IPatternStorage {
	val matterNode = SimpleMatterNode(patterns = this)

	private inner class Slot(container: SlottedContainer, slot: Int) : ContainerSlot(container, slot) {
		override val maxStackSize: Int
			get() = 1

		override fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
			return super.canAutomationPlaceItem(itemStack) && itemStack.getCapability(MatteryCapability.PATTERN_ITEM) != null
		}

		override fun canAutomationTakeItem(desired: Int): Boolean {
			return false
		}

		override fun notifyChanged(old: ItemStack) {
			if (!ItemStack.isSameItemSameComponents(item, old)) {
				if (!old.isEmpty) {
					old.getCapability(MatteryCapability.PATTERN_ITEM)?.let { cap: IPatternStorage ->
						cap.patterns.forEach { matterNode.graph.onPatternRemoved(it) }
					}
				}

				if (!item.isEmpty) {
					item.getCapability(MatteryCapability.PATTERN_ITEM)?.let { cap: IPatternStorage ->
						cap.patterns.forEach { matterNode.graph.onPatternAdded(it) }
					}
				}

				val level = level

				if (level is ServerLevel) {
					level.setBlock(blockPos, blockState.setValue(PatternStorageBlock.PATTERN_STORAGE_DISKS_PROPS[slot], item.getCapability(MatteryCapability.PATTERN_ITEM) != null), Block.UPDATE_CLIENTS)
				}
			}

			super.notifyChanged(old)
		}
	}

	val container = SlottedContainer.simple(2 * 4, ::Slot, ::markDirtyFast).also(::addDroppableContainer)
	val itemConfig = ConfigurableItemHandler(inputOutput = container)

	override fun setLevel(level: Level) {
		super.setLevel(level)
		matterNode.discover(this)
	}

	init {
		exposeGlobally(MatteryCapability.PATTERN_BLOCK, this)
		exposeGlobally(MatteryCapability.MATTER_NODE, matterNode)

		savetables.stateful(::container, INVENTORY_KEY)
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return PatternStorageMenu(containerID, inventory, this)
	}

	override val patterns: Stream<PatternState> get() {
		return container.stream()
			.filter { it.isNotEmpty }
			.map { it.getCapability(MatteryCapability.PATTERN_ITEM) }
			.filterNotNull()
			.flatMap { it.patterns }
	}

	override val patternCapacity: Int get() {
		val stored = container.sumOf { it.getCapability(MatteryCapability.PATTERN_ITEM)?.patternCapacity?.toLong() ?: 0L }
		return if (stored > Int.MAX_VALUE) Int.MAX_VALUE else stored.toInt()
	}

	override val storedPatterns: Int get() {
		val stored = container.sumOf { it.getCapability(MatteryCapability.PATTERN_ITEM)?.storedPatterns?.toLong() ?: 0L }
		return if (stored > Int.MAX_VALUE) Int.MAX_VALUE else stored.toInt()
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.isValid = false
	}

	override fun insertPattern(pattern: PatternState, onlyUpdate: Boolean, simulate: Boolean): PatternInsertStatus {
		for (spattern in this.container.iterator().map { it.getCapability(MatteryCapability.PATTERN_ITEM) }.filterNotNull()) {
			val status = spattern.insertPattern(pattern, onlyUpdate, simulate)

			if (!status.isFailed) {
				if (!simulate) {
					setChanged()

					if (status.isInserted) {
						matterNode.graph.onPatternAdded(status.newState!!)
					} else {
						matterNode.graph.onPatternUpdated(status.newState!!, status.oldState!!)
					}
				}

				return status
			}
		}

		return PatternInsertFailure
	}
}
