package ru.dbotthepony.mc.otm.config

import net.neoforged.neoforge.common.ModConfigSpec
import ru.dbotthepony.mc.otm.util.math.Decimal

class DecimalConfigValue(
	parent: ModConfigSpec.ConfigValue<String>,
	val minimum: Decimal? = null,
	val maximum: Decimal? = null,
) : ObservedConfigValue<Decimal>(parent) {
	override fun fromString(value: String): Decimal? {
		try {
			val parsed = Decimal(value)

			if (minimum != null && minimum > parsed) {
				return minimum
			} else if (maximum != null && maximum < parsed) {
				return maximum
			}

			return parsed
		} catch (err: java.lang.NumberFormatException) {
			return null
		}
	}

	override fun toString(value: Decimal): Pair<String, Decimal> {
		if (minimum != null && minimum > value) {
			return minimum.toString() to minimum
		} else if (maximum != null && maximum < value) {
			return maximum.toString() to maximum
		}

		return value.toString() to value
	}
}

private fun ModConfigSpec.Builder.commentRange(minimum: Decimal?, maximum: Decimal?) {
	if (minimum != null && maximum != null) {
		comment("Range: $minimum ~ $maximum")
	} else if (minimum != null) {
		comment("Range: >= $minimum")
	} else if (maximum != null) {
		comment("Range: <= $maximum")
	}
}

fun ModConfigSpec.Builder.defineDecimal(path: String, defaultValue: Decimal, minimum: Decimal? = null, maximum: Decimal? = null): DecimalConfigValue {
	commentRange(minimum, maximum)
	comment("Default: $defaultValue")
	return DecimalConfigValue(define(path, defaultValue.toString()), minimum, maximum)
}

fun ModConfigSpec.Builder.defineDecimal(path: List<String>, defaultValue: Decimal, minimum: Decimal? = null, maximum: Decimal? = null): DecimalConfigValue {
	commentRange(minimum, maximum)
	comment("Default: $defaultValue")
	return DecimalConfigValue(define(path, defaultValue.toString()), minimum, maximum)
}
