package ru.dbotthepony.mc.otm.config

import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.mc.otm.util.math.Decimal
import ru.dbotthepony.mc.otm.registry.MNames

object ItemsConfig : AbstractConfig("items") {
	init {
		builder.push("EnergyBatteries")
	}

	object Batteries {
		val CRUDE = batteryValues(MNames.BATTERY_CRUDE, Decimal(100_000), Decimal(160), Decimal(40), Decimal(80_000))
		val BASIC = batteryValues(MNames.BATTERY_BASIC, Decimal(400_000), Decimal(600))
		val NORMAL = batteryValues(MNames.BATTERY_NORMAL, Decimal(2_000_000), Decimal(1_000))
		val DENSE = batteryValues(MNames.BATTERY_DENSE, Decimal(10_000_000), Decimal(2_500))
		val CAPACITOR = batteryValues(MNames.BATTERY_CAPACITOR, Decimal(2_000_000), Decimal(50_000))

		val QUANTUM_BATTERY = conciseValues(MNames.QUANTUM_BATTERY, Decimal(40_000_000), Decimal(10_000))
		val QUANTUM_CAPACITOR = conciseValues(MNames.QUANTUM_CAPACITOR, Decimal(1_000_000), Decimal(200_000))

		val ZPM = conciseValues(MNames.ZPM_BATTERY, Decimal(200_000_000_000_000L), Decimal(200_000_000L))
	}

	init {
		Batteries
		builder.pop()
	}

	val MATTER_DUST_CAPACITY by builder.comment("Maximal matter value one matter dust item can have").defineDecimal("matterDustCapacity", Decimal(2_000), minimum = Decimal.ONE_TENTH)

	init {
		builder.push("MatterCapacitors")
	}

	object Capacitors {
		val BASIC by builder.defineDecimal(MNames.MATTER_CAPACITOR_BASIC, Decimal(2_500), minimum = Decimal.ONE_TENTH)
		val NORMAL by builder.defineDecimal(MNames.MATTER_CAPACITOR_NORMAL, Decimal(10_000), minimum = Decimal.ONE_TENTH)
		val DENSE by builder.defineDecimal(MNames.MATTER_CAPACITOR_DENSE, Decimal(40_000), minimum = Decimal.ONE_TENTH)
	}

	init {
		Capacitors
		builder.pop()
	}

	object PatternDrives {
		val NORMAL: Int by builder.defineInRange(MNames.PATTERN_DRIVE_NORMAL, 4, 1, Int.MAX_VALUE)
		val DOUBLE: Int by builder.defineInRange(MNames.PATTERN_DRIVE_DOUBLE, 8, 1, Int.MAX_VALUE)
		val TRIPLE: Int by builder.defineInRange(MNames.PATTERN_DRIVE_TRIPLE, 16, 1, Int.MAX_VALUE)
		val QUAD: Int by builder.defineInRange(MNames.PATTERN_DRIVE_QUAD, 32, 1, Int.MAX_VALUE)
	}

	init {
		builder.push("PatternDrives")
		PatternDrives
		builder.pop()
	}

	val FLUID_CAPSULE_CAPACITY: Int by builder.defineInRange("FLUID_CAPSULE_CAPACITY", 1000, 1, Int.MAX_VALUE)
	val FLUID_TANK_CAPACITY: Int by builder.defineInRange("FLUID_TANK_CAPACITY", 32_000, 1, Int.MAX_VALUE)
}
