package ru.dbotthepony.mc.otm.container

import net.minecraft.world.item.ItemStack
import net.neoforged.neoforge.items.IItemHandler

/**
 * Slot of [IAutomatedContainer], with additional methods to implement interaction behavior for both for players and mechanisms
 */
interface IAutomatedContainerSlot : IContainerSlot {
	fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
		return true
	}

	fun canAutomationTakeItem(desired: Int = item.count): Boolean {
		return true
	}

	fun modifyAutomationPlaceCount(itemStack: ItemStack): Int {
		return itemStack.count
	}

	fun modifyAutomationExtractionCount(desired: Int): Int {
		return desired
	}

	/**
	 * Slot-specific implementation for [IItemHandler.insertItem]
	 */
	fun insertItem(stack: ItemStack, simulate: Boolean): ItemStack {
		if (!canAutomationPlaceItem(stack))
			return stack

		var amount = modifyAutomationPlaceCount(stack)

		if (amount <= 0)
			return stack

		if (item.isEmpty) {
			amount = stack.count.coerceAtMost(maxStackSize(stack)).coerceAtMost(amount)

			if (!simulate) {
				item = stack.copyWithCount(amount)
			}

			if (stack.count <= amount) {
				return ItemStack.EMPTY
			} else {
				return stack.copyWithCount(stack.count - amount)
			}
		} else if (item.isStackable && maxStackSize(item) > item.count && ItemStack.isSameItemSameComponents(item, stack)) {
			val newCount = maxStackSize(item).coerceAtMost(item.count + stack.count.coerceAtMost(amount))
			val diff = newCount - item.count

			if (diff != 0) {
				if (!simulate) {
					item.grow(diff)
					setChanged()
				}

				val copy = stack.copy()
				copy.shrink(diff)
				return copy
			}
		}

		return stack
	}

	fun extractItem(amount: Int, simulate: Boolean): ItemStack {
		if (amount <= 0 || !canAutomationTakeItem(amount) || item.isEmpty)
			return ItemStack.EMPTY

		@Suppress("name_shadowing")
		val amount = modifyAutomationExtractionCount(amount)
		if (amount <= 0 || !canAutomationTakeItem(amount)) return ItemStack.EMPTY

		val minimal = amount.coerceAtMost(item.count)
		val copy = item.copy()
		copy.count = minimal

		if (!simulate) {
			if (item.count == minimal) {
				item = ItemStack.EMPTY
			} else {
				item.shrink(minimal)
			}

			setChanged()
		}

		return copy
	}
}
