package ru.dbotthepony.mc.otm.util

import net.minecraft.advancements.CriteriaTriggers
import net.minecraft.core.BlockPos
import net.minecraft.core.HolderLookup
import net.minecraft.core.registries.Registries
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.Tag
import net.minecraft.resources.ResourceKey
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerLevel
import net.minecraft.server.level.ServerPlayer
import net.minecraft.util.RandomSource
import net.minecraft.world.Container
import net.minecraft.world.Containers
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.storage.loot.LootParams
import net.minecraft.world.level.storage.loot.LootTable
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSets
import net.minecraft.world.level.storage.loot.parameters.LootContextParams
import net.minecraft.world.phys.Vec3
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.LOOT_TABLE_KEY
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.LOOT_TABLE_SEED_KEY

class BlockLootTableHolder(private val listener: Runnable = Runnable {  }) {
	private var ignoreListener = false

	var lootTable: ResourceKey<LootTable>? = null
		set(value) {
			if (field != value) {
				field = value
				if (!ignoreListener) listener.run()
			}
		}

	var lootTableSeed: Long? = null
		set(value) {
			if (field != value) {
				field = value
				if (!ignoreListener) listener.run()
			}
		}

	var lootTableRounds: Int? = null
		set(value) {
			if (field != value) {
				field = value
				if (!ignoreListener) listener.run()
			}
		}

	fun save(nbt: CompoundTag, registry: HolderLookup.Provider) {
		try {
			ignoreListener = true

			if (lootTable != null) {
				nbt.putString(LOOT_TABLE_KEY, lootTable!!.location().toString())

				if (lootTableSeed != null)
					nbt.putLong(LOOT_TABLE_SEED_KEY, lootTableSeed!!)

				if (lootTableRounds != null)
					nbt.putInt("LootRounds", lootTableRounds!!)
			}
		} finally {
			ignoreListener = false
		}
	}

	fun load(nbt: CompoundTag, registry: HolderLookup.Provider) {
		try {
			ignoreListener = true

			if (nbt.contains(LOOT_TABLE_KEY, Tag.TAG_STRING.toInt())) {
				lootTable = ResourceLocation.tryParse(nbt.getString(LOOT_TABLE_KEY))?.let { ResourceKey.create(Registries.LOOT_TABLE, it) }
				lootTableSeed = if (nbt.contains(LOOT_TABLE_SEED_KEY, Tag.TAG_LONG.toInt())) nbt.getLong(LOOT_TABLE_SEED_KEY) else null
				lootTableRounds = if (nbt.contains("LootRounds", Tag.TAG_INT.toInt())) nbt.getInt("LootRounds") else null

				if (lootTableRounds != null && lootTableRounds!! < 0)
					lootTableRounds = null
			}
		} finally {
			ignoreListener = false
		}
	}

	fun lookup(level: ServerLevel): LootTable {
		val lootTable = lootTable ?: return LootTable.EMPTY
		return level.server.reloadableRegistries().getLootTable(lootTable)
	}

	private fun selectRandom(overrideSeed: Long?, fallback: RandomSource): RandomSource {
		val lootTableSeed = overrideSeed ?: lootTableSeed
		return if (lootTableSeed == null) fallback else GJRAND64RandomSource(lootTableSeed)
	}

	fun getItems(params: LootParams, level: ServerLevel, overrideSeed: Long? = null, rounds: Int? = null): MutableList<ItemStack> {
		return lookup(level).getRandomItems(params, selectRandom(overrideSeed, level.otmRandom), rounds ?: lootTableRounds ?: 1)
	}

	fun getItems(params: LootParams.Builder, blockState: BlockState, overrideSeed: Long? = null, rounds: Int? = null): MutableList<ItemStack> {
		val level = params.level
		val create = params.withParameter(LootContextParams.BLOCK_STATE, blockState).create(LootContextParamSets.BLOCK)
		return getItems(create, level, overrideSeed, rounds)
	}

	fun fill(level: ServerLevel, blockPos: BlockPos, container: Container, ply: Player? = null, blockEntity: BlockEntity? = level.getBlockEntityNow(blockPos), overrideSeed: Long? = null, dropContents: Boolean = true, rounds: Int? = null) {
		if (rounds == 0)
			return
		else if (rounds != null && rounds < 0)
			throw IllegalArgumentException("Negative amount of rounds: $rounds")

		val lootTable = lootTable ?: return
		val server = level.server
		val loot = server.reloadableRegistries().getLootTable(lootTable)

		if (ply is ServerPlayer)
			CriteriaTriggers.GENERATE_LOOT.trigger(ply, lootTable)

		val params = LootParams.Builder(level)
			.withParameter(LootContextParams.ORIGIN, Vec3.atCenterOf(blockPos))

		if (ply != null) {
			params
				.withLuck(ply.luck)
				.withParameter(LootContextParams.THIS_ENTITY, ply)
		}

		if (blockEntity != null)
			params.withParameter(LootContextParams.BLOCK_ENTITY, blockEntity)

		if (dropContents)
			Containers.dropContents(level, blockPos, container)

		loot.fill(params.create(LootContextParamSets.CHEST), selectRandom(overrideSeed, level.otmRandom), container, rounds ?: lootTableRounds ?: 1)
	}

	fun clear() {
		lootTable = null
		lootTableSeed = null
	}
}
